@@ -14,4 +14,5 @@ autoconf
 (cd test; aclocal; automake --foreign; autoconf)
 
 # Run configure for this platform
-./configure $*
+#./configure $*
+echo "Now you are ready to run ./configure"
