@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL.c,v 1.11 2000/02/21 15:10:31 hercules Exp $";
+ "@(#) $Id: SDL.c,v 1.5.2.12 2000/09/07 07:52:14 hercules Exp $";
 #endif
 
 /* Initialization code for SDL */
@@ -32,9 +32,15 @@ static char rcsid =
 #include "SDL.h"
 #include "SDL_endian.h"
 #include "SDL_fatal.h"
+#ifndef DISABLE_VIDEO
 #include "SDL_leaks.h"
+#endif
 
 /* Initialization/Cleanup routines */
+#ifndef DISABLE_JOYSTICK
+extern int  SDL_JoystickInit(void);
+extern void SDL_JoystickQuit(void);
+#endif
 #ifndef DISABLE_CDROM
 extern int  SDL_CDROMInit(void);
 extern void SDL_CDROMQuit(void);
@@ -50,34 +56,15 @@ static SDL_version version =
 	{ SDL_MAJOR_VERSION, SDL_MINOR_VERSION, SDL_PATCHLEVEL };
 
 /* The initialized subsystems */
-Uint32 SDL_initialized = 0;
-static int ticks_started = 0;
+static Uint32 SDL_initialized = 0;
+static Uint32 ticks_started = 0;
 
 #ifdef CHECK_LEAKS
 int surfaces_allocated = 0;
 #endif
 
-int SDL_Init(Uint32 flags)
+int SDL_InitSubSystem(Uint32 flags)
 {
-#ifndef DISABLE_TIMERS
-	/* Initialize the timer subsystem */
-	if ( ! ticks_started ) {
-		SDL_StartTicks();
-		ticks_started = 1;
-	}
-	if ( (flags & SDL_INIT_TIMER) && !(SDL_initialized & SDL_INIT_TIMER) ) {
-		if ( SDL_TimerInit() < 0 ) {
-			return(-1);
-		}
-		SDL_initialized |= SDL_INIT_TIMER;
-	}
-#else
-	if ( flags & SDL_INIT_TIMER ) {
-		SDL_SetError("SDL not built with timer support");
-		return(-1);
-	}
-#endif
-
 #ifndef DISABLE_VIDEO
 	/* Initialize the video/event subsystem */
 	if ( (flags & SDL_INIT_VIDEO) && !(SDL_initialized & SDL_INIT_VIDEO) ) {
@@ -109,6 +96,41 @@ int SDL_Init(Uint32 flags)
 	}
 #endif
 
+#ifndef DISABLE_TIMERS
+	/* Initialize the timer subsystem */
+	if ( ! ticks_started ) {
+		SDL_StartTicks();
+		ticks_started = 1;
+	}
+	if ( (flags & SDL_INIT_TIMER) && !(SDL_initialized & SDL_INIT_TIMER) ) {
+		if ( SDL_TimerInit() < 0 ) {
+			return(-1);
+		}
+		SDL_initialized |= SDL_INIT_TIMER;
+	}
+#else
+	if ( flags & SDL_INIT_TIMER ) {
+		SDL_SetError("SDL not built with timer support");
+		return(-1);
+	}
+#endif
+
+#ifndef DISABLE_JOYSTICK
+	/* Initialize the joystick subsystem */
+	if ( (flags & SDL_INIT_JOYSTICK) &&
+	     !(SDL_initialized & SDL_INIT_JOYSTICK) ) {
+		if ( SDL_JoystickInit() < 0 ) {
+			return(-1);
+		}
+		SDL_initialized |= SDL_INIT_JOYSTICK;
+	}
+#else
+	if ( flags & SDL_INIT_JOYSTICK ) {
+		SDL_SetError("SDL not built with joystick support");
+		return(-1);
+	}
+#endif
+
 #ifndef DISABLE_CDROM
 	/* Initialize the CD-ROM subsystem */
 	if ( (flags & SDL_INIT_CDROM) && !(SDL_initialized & SDL_INIT_CDROM) ) {
@@ -123,6 +145,18 @@ int SDL_Init(Uint32 flags)
 		return(-1);
 	}
 #endif
+	return(0);
+}
+
+int SDL_Init(Uint32 flags)
+{
+	/* Clear the error message */
+	SDL_ClearError();
+
+	/* Initialize the desired subsystems */
+	if ( SDL_InitSubSystem(flags) < 0 ) {
+		return(-1);
+	}
 
 	/* Everything is initialized */
 	if ( !(flags & SDL_INIT_NOPARACHUTE) ) {
@@ -131,36 +165,56 @@ int SDL_Init(Uint32 flags)
 	return(0);
 }
 
-void SDL_Quit(void)
+void SDL_QuitSubSystem(Uint32 flags)
 {
-	/* Shut down all initialized subsystems */
-#ifndef DISABLE_TIMERS
-	if ( SDL_initialized & SDL_INIT_TIMER ) {
-		SDL_TimerQuit();
-		SDL_initialized &= ~SDL_INIT_TIMER;
-	}
-#endif
+	/* Shut down requested initialized subsystems */
 #ifndef DISABLE_CDROM
-	if ( SDL_initialized & SDL_INIT_CDROM ) {
+	if ( (flags & SDL_initialized & SDL_INIT_CDROM) ) {
 		SDL_CDROMQuit();
 		SDL_initialized &= ~SDL_INIT_CDROM;
 	}
 #endif
+#ifndef DISABLE_JOYSTICK
+	if ( (flags & SDL_initialized & SDL_INIT_JOYSTICK) ) {
+		SDL_JoystickQuit();
+		SDL_initialized &= ~SDL_INIT_JOYSTICK;
+	}
+#endif
+#ifndef DISABLE_TIMERS
+	if ( (flags & SDL_initialized & SDL_INIT_TIMER) ) {
+		SDL_TimerQuit();
+		SDL_initialized &= ~SDL_INIT_TIMER;
+	}
+#endif
 #ifndef DISABLE_AUDIO
-	if ( SDL_initialized & SDL_INIT_AUDIO ) {
+	if ( (flags & SDL_initialized & SDL_INIT_AUDIO) ) {
 		SDL_AudioQuit();
 		SDL_initialized &= ~SDL_INIT_AUDIO;
 	}
 #endif
 #ifndef DISABLE_VIDEO
-	if ( SDL_initialized & SDL_INIT_VIDEO ) {
+	if ( (flags & SDL_initialized & SDL_INIT_VIDEO) ) {
 		SDL_VideoQuit();
 		SDL_initialized &= ~SDL_INIT_VIDEO;
 	}
 #endif
-	SDL_ClearError();
+}
+
+Uint32 SDL_WasInit(Uint32 flags)
+{
+	if ( ! flags ) {
+		flags = SDL_INIT_EVERYTHING;
+	}
+	return (SDL_initialized&flags);
+}
+
+void SDL_Quit(void)
+{
+	/* Quit all subsystems */
+	SDL_QuitSubSystem(SDL_INIT_EVERYTHING);
 
 #ifdef CHECK_LEAKS
+	/* Print the number of surfaces not freed */
 	if ( surfaces_allocated != 0 ) {
 		fprintf(stderr, "SDL Warning: %d SDL surfaces extant\n", 
 							surfaces_allocated);
