@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_audio.c,v 1.25 2000/03/03 23:39:40 hercules Exp $";
+ "@(#) $Id: SDL_audio.c,v 1.7.2.28 2000/09/17 23:29:24 hercules Exp $";
 #endif
 
 /* Allow access to a raw mixing buffer */
@@ -60,8 +60,11 @@ static AudioBootStrap *bootstrap[] = {
 #ifdef __BEOS__
 	&BAUDIO_bootstrap,
 #endif
-#ifdef macintosh
+#if defined(macintosh) && !TARGET_API_MAC_CARBON
 	&AUDIO_bootstrap,
+#endif
+#ifdef _AIX
+	&Paud_bootstrap,
 #endif
 	NULL
 };
@@ -73,7 +76,7 @@ void SDL_AudioQuit(void);
 
 
 /* The general mixing thread function */
-static int RunAudio(void *audiop)
+int SDL_RunAudio(void *audiop)
 {
 	SDL_AudioDevice *audio = (SDL_AudioDevice *)audiop;
 	Uint8 *stream;
@@ -116,7 +119,12 @@ static int RunAudio(void *audiop)
 
 		/* Fill the current buffer with sound */
 		if ( audio->convert.needed ) {
-			stream = audio->convert.buf;
+			/* The buffer may not be allocated yet */
+			if ( audio->convert.buf ) {
+				stream = audio->convert.buf;
+			} else {
+				continue;
+			}
 		} else {
 			stream = audio->GetAudioBuf(audio);
 			if ( stream == NULL ) {
@@ -157,7 +165,7 @@ static int RunAudio(void *audiop)
 int SDL_AudioInit(const char *driver_name)
 {
 	SDL_AudioDevice *audio;
-	int i, idx;
+	int i = 0, idx;
 
 	/* Check to make sure we don't overwrite 'current_audio' */
 	if ( current_audio != NULL ) {
@@ -221,11 +229,17 @@ int SDL_AudioInit(const char *driver_name)
 		}
 		if ( audio == NULL ) {
 			SDL_SetError("No available audio device");
+#if 0 /* Don't fail SDL_Init() if audio isn't available.
+         SDL_OpenAudio() will handle it at that point.  *sigh*
+       */
 			return(-1);
+#endif
 		}
 	}
 	current_audio = audio;
-	current_audio->name = bootstrap[i]->name;
+	if ( current_audio ) {
+		current_audio->name = bootstrap[i]->name;
+	}
 	return(0);
 }
 
@@ -242,11 +256,11 @@ char *SDL_AudioDriverName(char *namebuf, int maxlen)
 int SDL_OpenAudio(SDL_AudioSpec *desired, SDL_AudioSpec *obtained)
 {
 	SDL_AudioDevice *audio;
-	int audio_opened;
 
 	/* Start up the audio driver, if necessary */
 	if ( ! current_audio ) {
-		if ( SDL_Init(SDL_INIT_AUDIO|SDL_INIT_NOPARACHUTE) < 0 ) {
+		if ( (SDL_InitSubSystem(SDL_INIT_AUDIO) < 0) ||
+		     (current_audio == NULL) ) {
 			return(-1);
 		}
 	}
@@ -286,8 +300,8 @@ int SDL_OpenAudio(SDL_AudioSpec *desired, SDL_AudioSpec *obtained)
 	audio->convert.needed = 0;
 	audio->enabled = 1;
 	audio->paused  = 1;
-	audio_opened = audio->OpenAudio(audio, &audio->spec);
-	if ( audio_opened < 0 ) {
+	audio->opened = audio->OpenAudio(audio, &audio->spec)+1;
+	if ( ! audio->opened ) {
 		SDL_CloseAudio();
 		return(-1);
 	}
@@ -340,10 +354,10 @@ int SDL_OpenAudio(SDL_AudioSpec *desired, SDL_AudioSpec *obtained)
 	}
 
 	/* Start the audio thread if necessary */
-	switch (audio_opened) {
-		case  0:
+	switch (audio->opened) {
+		case  1:
 			/* Start the audio thread */
-			audio->thread = SDL_CreateThread(RunAudio, audio);
+			audio->thread = SDL_CreateThread(SDL_RunAudio, audio);
 			if ( audio->thread == NULL ) {
 				SDL_CloseAudio();
 				SDL_SetError("Couldn't create audio thread");
@@ -358,6 +372,22 @@ int SDL_OpenAudio(SDL_AudioSpec *desired, SDL_AudioSpec *obtained)
 	return(0);
 }
 
+SDL_audiostatus SDL_GetAudioStatus(void)
+{
+	SDL_AudioDevice *audio = current_audio;
+	SDL_audiostatus status;
+
+	status = SDL_AUDIO_STOPPED;
+	if ( audio && audio->enabled ) {
+		if ( audio->paused ) {
+			status = SDL_AUDIO_PAUSED;
+		} else {
+			status = SDL_AUDIO_PLAYING;
+		}
+	}
+	return(status);
+}
+
 void SDL_PauseAudio (int pause_on)
 {
 	SDL_AudioDevice *audio = current_audio;
@@ -394,6 +424,11 @@ void SDL_UnlockAudio (void)
 }
 
 void SDL_CloseAudio (void)
+{
+	SDL_QuitSubSystem(SDL_INIT_AUDIO);
+}
+
+void SDL_AudioQuit(void)
 {
 	SDL_AudioDevice *audio = current_audio;
 
@@ -411,21 +446,15 @@ void SDL_CloseAudio (void)
 		if ( audio->convert.needed ) {
 			SDL_FreeAudioMem(audio->convert.buf);
 		}
-		audio->CloseAudio(audio);
-	}
-	SDL_AudioQuit();
-}
+		if ( audio->opened ) {
+			audio->CloseAudio(audio);
+			audio->opened = 0;
+		}
 
-void SDL_AudioQuit(void)
-{
-	/* HACK - this is done properly in SDL 1.1 */
-	extern Uint32 SDL_initialized;
-
-	if ( current_audio ) {
-		current_audio->free(current_audio);
+		/* Free the driver data */
+		audio->free(audio);
 		current_audio = NULL;
 	}
-	SDL_initialized &= ~SDL_INIT_AUDIO;
 }
 
 #define NUM_FORMATS	6
