@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_syscdrom.c,v 1.3 2000/02/29 23:13:55 hercules Exp $";
+ "@(#) $Id: SDL_syscdrom.c,v 1.2.2.4 2000/03/30 21:47:55 hercules Exp $";
 #endif
 
 /* Functions for system-level CD-ROM audio control */
@@ -30,6 +30,7 @@ static char rcsid =
 #include <sys/types.h>
 #include <stdlib.h>
 #include <sys/stat.h>
+#include <sys/ioctl.h>
 #include <fcntl.h>
 #include <stdio.h>
 #include <string.h>
@@ -122,11 +123,13 @@ static int CheckDrive(char *drive, char *mnttype, struct stat *stbuf)
 				is_cd = 1;
 			}
 			close(cdfd);
-		} else
+		}
+#ifdef USE_MNTENT
 		/* Even if we can't read it, it might be mounted */
-		if ( mnttype && (strcmp(mnttype, MNTTYPE_CDROM) == 0) ) {
+		else if ( mnttype && (strcmp(mnttype, MNTTYPE_CDROM) == 0) ) {
 			is_cd = 1;
 		}
+#endif
 	}
 	return(is_cd);
 }
