@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_events_c.h,v 1.5 2000/01/22 00:55:56 hercules Exp $";
+ "@(#) $Id: SDL_events_c.h,v 1.3.2.5 2000/04/28 22:45:04 hercules Exp $";
 #endif
 
 /* Useful functions and variables from SDL_events.c */
@@ -48,8 +48,6 @@ extern SDL_EventFilter SDL_EventOK;
 
 /* The array of event processing states */
 extern Uint8 SDL_ProcessEvents[SDL_NUMEVENTS];
-extern void SDL_PushEvent(SDL_Event *event);
-extern int SDL_CheckQuit(SDL_Event *event);
 
 /* Internal event queueing functions
    (from SDL_active.c, SDL_mouse.c, SDL_keyboard.c, SDL_quit.c, SDL_events.c)
@@ -59,6 +57,7 @@ extern int SDL_PrivateMouseMotion(Uint8 buttonstate, int relative,
 						Sint16 x, Sint16 y);
 extern int SDL_PrivateMouseButton(Uint8 state, Uint8 button,Sint16 x,Sint16 y);
 extern int SDL_PrivateKeyboard(Uint8 state, SDL_keysym *key);
+extern int SDL_PrivateResize(int w, int h);
 extern int SDL_PrivateQuit(void);
 extern int SDL_PrivateSysWMEvent(SDL_SysWMmsg *message);
 
