@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_thread.c,v 1.3 2000/03/03 00:24:35 hercules Exp $";
+ "@(#) $Id: SDL_thread.c,v 1.2.2.8 2000/10/23 04:52:03 hercules Exp $";
 #endif
 
 /* System independent thread management routines for SDL */
@@ -46,13 +46,20 @@ static int SDL_maxthreads = 0;
 static int SDL_numthreads = 0;
 static SDL_Thread **SDL_Threads = NULL;
 static SDL_mutex *thread_lock = NULL;
+int _creating_thread_lock = 0;
 
 int SDL_ThreadsInit(void)
 {
 	int retval;
 
 	retval = 0;
+	/* Set the thread lock creation flag so that we can reuse an
+	   existing lock on the system - since this mutex never gets
+	   destroyed (see SDL_ThreadsQuit()), we want to reuse it.
+	*/
+	_creating_thread_lock = 1;
 	thread_lock = SDL_CreateMutex();
+	_creating_thread_lock = 0;
 	if ( thread_lock == NULL ) {
 		retval = -1;
 	}
@@ -129,11 +136,11 @@ static void SDL_DelThread(SDL_Thread *thread)
 			}
 		}
 		if ( i < SDL_numthreads ) {
+			--SDL_numthreads;
 			while ( i < SDL_numthreads ) {
 				SDL_Threads[i] = SDL_Threads[i+1];
 				++i;
 			}
-			--SDL_numthreads;
 #ifdef DEBUG_THREADS
 			printf("Deleting thread (%d left - %d max)\n",
 					SDL_numthreads, SDL_maxthreads);
@@ -175,7 +182,7 @@ typedef struct {
 	int (*func)(void *);
 	void *data;
 	SDL_Thread *info;
-	SDL_mutex *wait;
+	SDL_sem *wait;
 } thread_args;
 
 void SDL_RunThread(void *data)
@@ -200,7 +207,7 @@ void SDL_RunThread(void *data)
 	statusloc = &args->info->status;
 
 	/* Wake up the parent thread */
-	SDL_mutexV(args->wait);
+	SDL_SemPost(args->wait);
 
 	/* Run the function */
 	*statusloc = userfunc(userdata);
@@ -231,14 +238,12 @@ SDL_Thread *SDL_CreateThread(int (*fn)(void *), void *data)
 	args->func = fn;
 	args->data = data;
 	args->info = thread;
-	args->wait = SDL_CreateMutex();
+	args->wait = SDL_CreateSemaphore(0);
 	if ( args->wait == NULL ) {
 		free(thread);
 		free(args);
-		SDL_OutOfMemory();
 		return(NULL);
 	}
-	SDL_mutexP(args->wait);		/* We will block later */
 
 	/* Add the thread to the list of available threads */
 	SDL_AddThread(thread);
@@ -247,14 +252,14 @@ SDL_Thread *SDL_CreateThread(int (*fn)(void *), void *data)
 	ret = SDL_SYS_CreateThread(thread, args);
 	if ( ret >= 0 ) {
 		/* Wait for the thread function to use arguments */
-		SDL_mutexP(args->wait);
+		SDL_SemWait(args->wait);
 	} else {
 		/* Oops, failed.  Gotta free everything */
 		SDL_DelThread(thread);
 		free(thread);
 		thread = NULL;
 	}
-	SDL_DestroyMutex(args->wait);
+	SDL_DestroySemaphore(args->wait);
 	free(args);
 
 	/* Everything is running now */
@@ -273,6 +278,18 @@ void SDL_WaitThread(SDL_Thread *thread, int *status)
 	}
 }
 
+Uint32 SDL_GetThreadID(SDL_Thread *thread)
+{
+	Uint32 id;
+
+	if ( thread ) {
+		id = thread->threadid;
+	} else {
+		id = SDL_ThreadID();
+	}
+	return(id);
+}
+
 void SDL_KillThread(SDL_Thread *thread)
 {
 	if ( thread ) {
