@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_pixels.c,v 1.3 1999/12/18 23:30:14 hercules Exp $";
+ "@(#) $Id: SDL_pixels.c,v 1.3.2.11 2000/09/25 06:14:38 hercules Exp $";
 #endif
 
 /* General (mostly internal) pixel/color manipulation routines for SDL */
@@ -39,12 +39,6 @@ static char rcsid =
 #include "SDL_pixels_c.h"
 #include "SDL_RLEaccel_c.h"
 
-/* We should probably define this ;-) */
-struct map_list {
-	SDL_BlitMap *map;
-	struct map_list *next;
-};
-
 /* Helper functions */
 /*
  * Allocate a pixel format structure and fill it according to the given info.
@@ -56,12 +50,13 @@ SDL_PixelFormat *SDL_AllocFormat(int bpp,
 	Uint32 mask;
 
 	/* Allocate an empty pixel format structure */
-	format = (SDL_PixelFormat *)malloc(sizeof(*format));
+	format = malloc(sizeof(*format));
 	if ( format == NULL ) {
 		SDL_OutOfMemory();
 		return(NULL);
 	}
 	memset(format, 0, sizeof(*format));
+	format->alpha = SDL_ALPHA_OPAQUE;
 
 	/* Set up the format */
 	format->BitsPerPixel = bpp;
@@ -235,22 +230,23 @@ SDL_PixelFormat *SDL_AllocFormat(int bpp,
 	}
 	return(format);
 }
+SDL_PixelFormat *SDL_ReallocFormat(SDL_Surface *surface, int bpp,
+			Uint32 Rmask, Uint32 Gmask, Uint32 Bmask, Uint32 Amask)
+{
+	if ( surface->format ) {
+		SDL_FreeFormat(surface->format);
+		SDL_FormatChanged(surface);
+	}
+	surface->format = SDL_AllocFormat(bpp, Rmask, Gmask, Bmask, Amask);
+	return surface->format;
+}
+
 /*
  * Change any previous mappings from/to the new surface format
  */
 void SDL_FormatChanged(SDL_Surface *surface)
 {
-	struct map_list *mapped, *freeme;
-
-	/* Mark all mappings invalid */
-	mapped = surface->mapped;
-	surface->mapped = NULL;
-	while ( mapped ) {
-		SDL_InvalidateMap(mapped->map);
-		freeme = mapped;
-		mapped = mapped->next;
-		free(freeme);
-	}
+	surface->format_version++;
 	SDL_InvalidateMap(surface->map);
 }
 /*
@@ -273,7 +269,7 @@ void SDL_FreeFormat(SDL_PixelFormat *format)
  */
 void SDL_DitherColors(SDL_Color *colors, int bpp)
 {
-	Uint8 r, g, b, i;
+	int r, g, b, i;
 
 	/* Calculate the dither values for each index */
 	switch (bpp) {
@@ -345,32 +341,96 @@ Uint8 SDL_FindColor(SDL_Palette *pal, Uint8 r, Uint8 g, Uint8 b)
 	return(pixel);
 }
 
-/* Map from RGB to a 32bit pixel */
-Uint32 SDL_MapRGB (SDL_PixelFormat *format, Uint8 r, Uint8 g, Uint8 b)
+/* Find the opaque pixel value corresponding to an RGB triple */
+Uint32 SDL_MapRGB(SDL_PixelFormat *format, Uint8 r, Uint8 g, Uint8 b)
 {
-	Uint32 pixel;
-
 	if ( format->palette == NULL ) {
-		pixel = ((r>>format->Rloss)<<format->Rshift)|
-			((g>>format->Gloss)<<format->Gshift)|
-			((b>>format->Bloss)<<format->Bshift);
+		return (r >> format->Rloss) << format->Rshift
+		       | (g >> format->Gloss) << format->Gshift
+		       | (b >> format->Bloss) << format->Bshift
+		       | format->Amask;
 	} else {
-		pixel = (Uint32)SDL_FindColor(format->palette, r, g, b);
+		return SDL_FindColor(format->palette, r, g, b);
 	}
-	return(pixel);
 }
+
+/* Find the pixel value corresponding to an RGBA quadruple */
+Uint32 SDL_MapRGBA(SDL_PixelFormat *format, Uint8 r, Uint8 g, Uint8 b, Uint8 a)
+{
+	if ( format->palette == NULL ) {
+	        return (r >> format->Rloss) << format->Rshift
+		    | (g >> format->Gloss) << format->Gshift
+		    | (b >> format->Bloss) << format->Bshift
+		    | ((a >> format->Aloss) << format->Ashift & format->Amask);
+	} else {
+		return SDL_FindColor(format->palette, r, g, b);
+	}
+}
+
+void SDL_GetRGBA(Uint32 pixel, SDL_PixelFormat *fmt,
+		 Uint8 *r, Uint8 *g, Uint8 *b, Uint8 *a)
+{
+	if ( fmt->palette == NULL ) {
+	        /*
+		 * This makes sure that the result is mapped to the
+		 * interval [0..255], and the maximum value for each
+		 * component is 255. This is important to make sure
+		 * that white is indeed reported as (255, 255, 255),
+		 * and that opaque alpha is 255.
+		 * This only works for RGB bit fields at least 4 bit
+		 * wide, which is almost always the case.
+		 */
+	        unsigned rv, gv, bv, av;
+		rv = (pixel & fmt->Rmask) >> fmt->Rshift;
+		*r = (rv << fmt->Rloss) + (rv >> (8 - fmt->Rloss));
+		gv = (pixel & fmt->Gmask) >> fmt->Gshift;
+		*g = (gv << fmt->Gloss) + (gv >> (8 - fmt->Gloss));
+		bv = (pixel & fmt->Bmask) >> fmt->Bshift;
+		*b = (bv << fmt->Bloss) + (bv >> (8 - fmt->Bloss));
+		if(fmt->Amask) {
+		        av = (pixel & fmt->Amask) >> fmt->Ashift;
+			*a = (av << fmt->Aloss) + (av >> (8 - fmt->Aloss));
+		} else
+		        *a = SDL_ALPHA_OPAQUE;
+	} else {
+		*r = fmt->palette->colors[pixel].r;
+		*g = fmt->palette->colors[pixel].g;
+		*b = fmt->palette->colors[pixel].b;
+		*a = SDL_ALPHA_OPAQUE;
+	}
+}
+
 void SDL_GetRGB(Uint32 pixel, SDL_PixelFormat *fmt, Uint8 *r,Uint8 *g,Uint8 *b)
 {
 	if ( fmt->palette == NULL ) {
-		*r = (((pixel&fmt->Rmask)>>fmt->Rshift)<<fmt->Rloss);
-		*g = (((pixel&fmt->Gmask)>>fmt->Gshift)<<fmt->Gloss);
-		*b = (((pixel&fmt->Bmask)>>fmt->Bshift)<<fmt->Bloss);
+	        /* the note for SDL_GetRGBA above applies here too */
+	        unsigned rv, gv, bv;
+		rv = (pixel & fmt->Rmask) >> fmt->Rshift;
+		*r = (rv << fmt->Rloss) + (rv >> (8 - fmt->Rloss));
+		gv = (pixel & fmt->Gmask) >> fmt->Gshift;
+		*g = (gv << fmt->Gloss) + (gv >> (8 - fmt->Gloss));
+		bv = (pixel & fmt->Bmask) >> fmt->Bshift;
+		*b = (bv << fmt->Bloss) + (bv >> (8 - fmt->Bloss));
 	} else {
 		*r = fmt->palette->colors[pixel].r;
 		*g = fmt->palette->colors[pixel].g;
 		*b = fmt->palette->colors[pixel].b;
 	}
 }
+
+/* Apply gamma to a set of colors - this is easy. :) */
+void SDL_ApplyGamma(Uint8 *gamma, SDL_Color *colors, SDL_Color *output,
+							int ncolors)
+{
+	int i;
+
+	for ( i=0; i<ncolors; ++i ) {
+		output[i].r = gamma[0*256 + colors[i].r];
+		output[i].g = gamma[1*256 + colors[i].g];
+		output[i].b = gamma[2*256 + colors[i].b];
+	}
+}
+
 /* Map from Palette to Palette */
 static Uint8 *Map1to1(SDL_Palette *src, SDL_Palette *dst, int *identical)
 {
@@ -404,7 +464,8 @@ static Uint8 *Map1toN(SDL_Palette *src, SDL_PixelFormat *dst)
 {
 	Uint8 *map;
 	int i;
-	Uint8  bpp;
+	int  bpp;
+	unsigned alpha;
 
 	bpp = ((dst->BytesPerPixel == 3) ? 4 : dst->BytesPerPixel);
 	map = (Uint8 *)malloc(src->ncolors*bpp);
@@ -413,10 +474,12 @@ static Uint8 *Map1toN(SDL_Palette *src, SDL_PixelFormat *dst)
 		return(NULL);
 	}
 
+	alpha = dst->Amask ? SDL_ALPHA_OPAQUE : 0;
 	/* We memory copy to the pixel map so the endianness is preserved */
 	for ( i=0; i<src->ncolors; ++i ) {
-		ASSEMBLE_RGB(&map[i*bpp], dst->BytesPerPixel, dst,
-			src->colors[i].r, src->colors[i].g, src->colors[i].b);
+		ASSEMBLE_RGBA(&map[i*bpp], dst->BytesPerPixel, dst,
+			      src->colors[i].r, src->colors[i].g,
+			      src->colors[i].b, alpha);
 	}
 	return(map);
 }
@@ -459,26 +522,11 @@ SDL_BlitMap *SDL_AllocBlitMap(void)
 }
 void SDL_InvalidateMap(SDL_BlitMap *map)
 {
-	if ( map->dst ) {
-		struct map_list *mapped, *prev;
-
-		/* Remove this mapping from the destination's map list */
-		prev = NULL;
-		for ( mapped=map->dst->mapped; mapped; mapped=mapped->next ) {
-			if ( mapped->map == map ) {
-				if ( prev ) {
-					prev->next = mapped->next;
-				} else {
-					map->dst->mapped = mapped->next;
-				}
-				free(mapped);
-				break;
-			}
-			prev = mapped;
-		}
-		/* Mark the mapping as invalid */
-		map->dst = NULL;
+	if ( ! map ) {
+		return;
 	}
+	map->dst = NULL;
+	map->format_version = (unsigned int)-1;
 	if ( map->table ) {
 		free(map->table);
 		map->table = NULL;
@@ -493,7 +541,7 @@ int SDL_MapSurface (SDL_Surface *src, SDL_Surface *dst)
 	/* Clear out any previous mapping */
 	map = src->map;
 	if ( (src->flags & SDL_RLEACCEL) == SDL_RLEACCEL ) {
-		SDL_UnRLESurface(src);
+		SDL_UnRLESurface(src, 1);
 	}
 	SDL_InvalidateMap(map);
 
@@ -554,29 +602,19 @@ int SDL_MapSurface (SDL_Surface *src, SDL_Surface *dst)
 		break;
 	}
 
-	/* Add this mapping to the destination's map list */
-	{
-		struct map_list *mapped;
-
-		mapped = (struct map_list *)malloc(sizeof(*mapped));
-		if ( mapped == NULL ) {
-			SDL_OutOfMemory();
-			return(-1);
-		}
-		mapped->map = map;
-		mapped->next = dst->mapped;
-		dst->mapped = mapped;
-	}
 	map->dst = dst;
+	map->format_version = dst->format_version;
 
 	/* Choose your blitters wisely */
 	return(SDL_CalculateBlit(src));
 }
 void SDL_FreeBlitMap(SDL_BlitMap *map)
 {
-	SDL_InvalidateMap(map);
-	if ( map->sw_data != NULL ) {
-		free(map->sw_data);
+	if ( map ) {
+		SDL_InvalidateMap(map);
+		if ( map->sw_data != NULL ) {
+			free(map->sw_data);
+		}
+		free(map);
 	}
-	free(map);
 }
