@@ -1,6 +1,6 @@
 /*
 	SDL - Simple DirectMedia Layer
-	Copyright (C) 1997, 1998, 1999  Sam Lantinga
+	Copyright (C) 1997, 1998, 1999, 2000  Sam Lantinga
 
 	This library is free software; you can redistribute it and/or
 	modify it under the terms of the GNU Library General Public
@@ -22,13 +22,14 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_fbevents.c,v 1.6 2000/01/25 00:14:41 hercules Exp $";
+ "@(#) $Id: SDL_fbevents.c,v 1.5.2.9 2000/09/28 20:10:39 hercules Exp $";
 #endif
 
 /* Handle the event stream, converting X11 events into SDL events */
 
 #include <sys/types.h>
 #include <sys/time.h>
+#include <sys/ioctl.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <unistd.h>
@@ -40,7 +41,6 @@ static char rcsid =
 #include <dirent.h>
 #include <ctype.h>
 
-#include "SDL_fbkeys.h"
 #include <linux/vt.h>
 #include <linux/kd.h>
 #include <linux/keyboard.h>
@@ -52,6 +52,7 @@ static char rcsid =
 #include "SDL_events_c.h"
 #include "SDL_fbvideo.h"
 #include "SDL_fbevents_c.h"
+#include "SDL_fbkeys.h"
 
 #ifndef GPM_NODE_FIFO
 #define GPM_NODE_FIFO	"/dev/gpmdata"
@@ -61,6 +62,7 @@ static char rcsid =
 #define NUM_VGAKEYMAPS	(1<<KG_CAPSSHIFT)
 static Uint16 vga_keymap[NUM_VGAKEYMAPS][NR_KEYS];
 static SDLKey keymap[128];
+static Uint16 keymap_temp[128]; /* only used at startup */
 static SDL_keysym *TranslateKey(int scancode, SDL_keysym *keysym);
 
 /* Ugh, we have to duplicate the kernel's keysym mapping code...
@@ -80,6 +82,10 @@ static void FB_vgainitkeymaps(int fd)
 			entry.kb_table = map;
 			entry.kb_index = i;
 			if ( ioctl(fd, KDGKBENT, &entry) == 0 ) {
+				/* fill keytemp. This replaces SDL_fbkeys.h */
+				if ( (map == 0) && (i<128) ) {
+					keymap_temp[i] = entry.kb_value;
+				}
 				/* The "Enter" key is a special case */
 				if ( entry.kb_value == K_ENTER ) {
 					entry.kb_value = K(KT_ASCII,13);
@@ -155,7 +161,11 @@ void FB_CloseKeyboard(_THIS)
 
 int FB_OpenKeyboard(_THIS)
 {
-	keyboard_fd = 0;
+	if ((keyboard_fd = open("/dev/tty", O_RDWR)) < 0) {
+		perror("Can't open /dev/tty\n");
+		keyboard_fd = -1;
+		return(-1);
+	}	
 	if ( keyboard_fd >= 0 ) {
 		struct termios keyboard_termios;
 
@@ -203,6 +213,7 @@ static enum {
 	MOUSE_NONE = -1,
 	MOUSE_GPM,	/* Note: GPM uses the MSC protocol */
 	MOUSE_PS2,
+	MOUSE_IMPS2,
 	MOUSE_MS,
 	MOUSE_BM,
 	NUM_MOUSE_DRVS
@@ -285,6 +296,64 @@ static int gpm_available(void)
 	return available;
 }
 
+/* Returns true if the mouse uses the IMPS/2 protocol */
+static int detect_imps2(int fd)
+{
+	int imps2;
+
+	imps2 = 0;
+	if ( getenv("SDL_MOUSEDEV_IMPS2") ) {
+		imps2 = 1;
+	}
+	if ( ! imps2 ) {
+		unsigned char query_ps2 = 0xF2;
+		fd_set fdset;
+		struct timeval tv;
+
+
+		/* Get rid of any mouse motion noise */
+		FD_ZERO(&fdset);
+		FD_SET(fd, &fdset);
+		tv.tv_sec = 0;
+		tv.tv_usec = 0;
+		while ( select(fd+1, &fdset, 0, 0, &tv) > 0 ) {
+			char temp[32];
+			read(fd, temp, sizeof(temp));
+		}
+
+		/* Query for the type of mouse protocol */
+		if ( write(fd, &query_ps2, 1) == 1 ) {
+			unsigned char ch = 0;
+
+			/* If you wanted to control the mouse mode:
+			   Set IMPS/2 protocol:
+				{0xf3,200,0xf3,100,0xf3,80}
+			   Reset mouse device:
+				{0xFF}
+			*/
+			/* Get the mouse protocol response */
+			do {
+				FD_ZERO(&fdset);
+				FD_SET(fd, &fdset);
+				tv.tv_sec = 1;
+				tv.tv_usec = 0;
+				if ( select(fd+1, &fdset, 0, 0, &tv) < 1 ) {
+					break;
+				}
+			} while ( (read(fd, &ch, 1) == 1) &&
+			          ((ch == 0xFA) || (ch == 0xAA)) );
+
+			/* Experimental values (Logitech wheelmouse) */
+#ifdef DEBUG_MOUSE
+fprintf(stderr, "Last mouse mode: 0x%x\n", ch);
+#endif
+			if ( ch == 3 ) {
+				imps2 = 1;
+			}
+		}
+	}
+	return imps2;
+}
 
 int FB_OpenMouse(_THIS)
 {
@@ -307,12 +376,22 @@ fprintf(stderr, "Using GPM mouse\n");
 		}
 		/* First try to use a modern PS/2 port mouse */
 		if ( mouse_fd < 0 ) {
-			mouse_fd = open("/dev/psaux", O_RDONLY, 0);
+			mouse_fd = open("/dev/psaux", O_RDWR, 0);
+			if ( mouse_fd < 0 ) {
+				mouse_fd = open("/dev/psaux", O_RDONLY, 0);
+			}
 			if ( mouse_fd >= 0 ) {
+				if ( detect_imps2(mouse_fd) ) {
+#ifdef DEBUG_MOUSE
+fprintf(stderr, "Using IMPS/2 mouse\n");
+#endif
+					mouse_drv = MOUSE_IMPS2;
+				} else {
 #ifdef DEBUG_MOUSE
 fprintf(stderr, "Using PS/2 mouse\n");
 #endif
-				mouse_drv = MOUSE_PS2;
+					mouse_drv = MOUSE_PS2;
+				}
 			}
 		}
 		/* Next try to use a PPC ADB port mouse */
@@ -363,34 +442,33 @@ static int posted = 0;
 
 void FB_vgamousecallback(int button, int dx, int dy)
 {
+	int button_1, button_3;
+	int button_state;
+	int state_changed;
+	int i;
+	Uint8 state;
+
 	if ( dx || dy ) {
 		posted += SDL_PrivateMouseMotion(0, 1, dx, dy);
 	}
-	if ( button & 0x04 ) {  /* Left mouse button */
-		if ( !(SDL_GetMouseState(NULL, NULL) & SDL_BUTTON(1)) ) {
-			posted += SDL_PrivateMouseButton(SDL_PRESSED, 1, 0, 0);
-		}
-	} else {
-		if ( (SDL_GetMouseState(NULL, NULL) & SDL_BUTTON(1)) ) {
-			posted += SDL_PrivateMouseButton(SDL_RELEASED, 1, 0, 0);
-		}
-	}
-	if ( button & 0x02 ) {  /* Middle mouse button */
-		if ( !(SDL_GetMouseState(NULL, NULL) & SDL_BUTTON(2)) ) {
-			posted += SDL_PrivateMouseButton(SDL_PRESSED, 2, 0, 0);
-		}
-	} else {
-		if ( (SDL_GetMouseState(NULL, NULL) & SDL_BUTTON(2)) ) {
-			posted += SDL_PrivateMouseButton(SDL_RELEASED, 2, 0, 0);
-		}
-	}
-	if ( button & 0x01 ) {  /* Right mouse button */
-		if ( !(SDL_GetMouseState(NULL, NULL) & SDL_BUTTON(3)) ) {
-			posted += SDL_PrivateMouseButton(SDL_PRESSED, 3, 0, 0);
-		}
-	} else {
-		if ( (SDL_GetMouseState(NULL, NULL) & SDL_BUTTON(3)) ) {
-			posted += SDL_PrivateMouseButton(SDL_RELEASED, 3, 0, 0);
+
+	/* Swap button 1 and 3 */
+	button_1 = (button & 0x04) >> 2;
+	button_3 = (button & 0x01) << 2;
+	button &= ~0x05;
+	button |= (button_1|button_3);
+
+	/* See what changed */
+	button_state = SDL_GetMouseState(NULL, NULL);
+	state_changed = button_state ^ button;
+	for ( i=0; i<8; ++i ) {
+		if ( state_changed & (1<<i) ) {
+			if ( button & (1<<i) ) {
+				state = SDL_PRESSED;
+			} else {
+				state = SDL_RELEASED;
+			}
+			posted += SDL_PrivateMouseButton(state, i+1, 0, 0);
 		}
 	}
 }
@@ -403,9 +481,9 @@ static void handle_mouse(_THIS)
 	static int start = 0;
 	static unsigned char mousebuf[BUFSIZ];
 	int i, nread;
-	int button;
-	int dx, dy;
-	int packetsize;
+	int button = 0;
+	int dx = 0, dy = 0;
+	int packetsize = 0;
 
 	/* Figure out the mouse packet size */
 	switch (mouse_drv) {
@@ -416,11 +494,18 @@ static void handle_mouse(_THIS)
 		case MOUSE_GPM:
 			packetsize = 5;
 			break;
+		case MOUSE_IMPS2:
+			packetsize = 4;
+			break;
 		case MOUSE_PS2:
 		case MOUSE_MS:
 		case MOUSE_BM:
 			packetsize = 3;
 			break;
+		case NUM_MOUSE_DRVS:
+			/* Uh oh.. */
+			packetsize = 0;
+			break;
 	}
 
 	/* Read as many packets as possible */
@@ -466,6 +551,29 @@ static void handle_mouse(_THIS)
 		  		dy = (mousebuf[i] & 0x20) ?
 		  		      -(mousebuf[i+2] - 256) : -mousebuf[i+2];
 				break;
+			case MOUSE_IMPS2:
+				/* Get current mouse state */
+				button = (mousebuf[i] & 0x04) >> 1 | /*Middle*/
+		  			 (mousebuf[i] & 0x02) >> 1 | /*Right*/
+		  			 (mousebuf[i] & 0x01) << 2 | /*Left*/
+		  			 (mousebuf[i] & 0x40) >> 3 | /* 4 */
+		  			 (mousebuf[i] & 0x80) >> 3;  /* 5 */
+		  		dx = (mousebuf[i] & 0x10) ?
+		  		      mousebuf[i+1] - 256 : mousebuf[i+1];
+		  		dy = (mousebuf[i] & 0x20) ?
+		  		      -(mousebuf[i+2] - 256) : -mousebuf[i+2];
+				switch (mousebuf[i+3]&0x0F) {
+				    case 0x0E: /* DX = +1 */
+				    case 0x02: /* DX = -1 */
+					break;
+				    case 0x0F: /* DY = +1 (map button 4) */
+					button |= (1<<3);
+					break;
+				    case 0x01: /* DY = -1 (map button 5) */
+					button |= (1<<4);
+					break;
+				}
+				break;
 			case MOUSE_MS:
 				/* Microsoft protocol has 0x40 in high byte */
 				if ( (mousebuf[i] & 0x40) != 0x40 ) {
@@ -475,7 +583,7 @@ static void handle_mouse(_THIS)
 				}
 				/* Get current mouse state */
 				button = ((mousebuf[i] & 0x20) >> 3) |
-					     ((mousebuf[i] & 0x10) >> 4);
+				         ((mousebuf[i] & 0x10) >> 4);
 				dx = (signed char)(((mousebuf[i] & 0x03) << 6) |
 				                   (mousebuf[i + 1] & 0x3F));
 				dy = (signed char)(((mousebuf[i] & 0x0C) << 4) |
@@ -493,6 +601,11 @@ static void handle_mouse(_THIS)
 				dx =  (signed char)mousebuf[i+1];
 				dy = -(signed char)mousebuf[i+2];
 				break;
+			case NUM_MOUSE_DRVS:
+				/* Uh oh.. */
+				dx = 0;
+				dy = 0;
+				break;
 		}
 		FB_vgamousecallback(button, dx, dy);
 	}
@@ -641,126 +754,117 @@ void FB_InitOSKeymap(_THIS)
 {
 	int i;
 
-	/* Initialize the BeOS key translation table */
-	for ( i=0; i<SDL_TABLESIZE(keymap); ++i )
-		keymap[i] = SDLK_UNKNOWN;
+	/* Initialize the Linux key translation table */
 
-	keymap[SCANCODE_ESCAPE] = SDLK_ESCAPE;
-	keymap[SCANCODE_1] = SDLK_1;
-	keymap[SCANCODE_2] = SDLK_2;
-	keymap[SCANCODE_3] = SDLK_3;
-	keymap[SCANCODE_4] = SDLK_4;
-	keymap[SCANCODE_5] = SDLK_5;
-	keymap[SCANCODE_6] = SDLK_6;
-	keymap[SCANCODE_7] = SDLK_7;
-	keymap[SCANCODE_8] = SDLK_8;
-	keymap[SCANCODE_9] = SDLK_9;
-	keymap[SCANCODE_0] = SDLK_0;
-	keymap[SCANCODE_MINUS] = SDLK_MINUS;
-	keymap[SCANCODE_EQUAL] = SDLK_EQUALS;
-	keymap[SCANCODE_BACKSPACE] = SDLK_BACKSPACE;
-	keymap[SCANCODE_TAB] = SDLK_TAB;
-	keymap[SCANCODE_Q] = SDLK_q;
-	keymap[SCANCODE_W] = SDLK_w;
-	keymap[SCANCODE_E] = SDLK_e;
-	keymap[SCANCODE_R] = SDLK_r;
-	keymap[SCANCODE_T] = SDLK_t;
-	keymap[SCANCODE_Y] = SDLK_y;
-	keymap[SCANCODE_U] = SDLK_u;
-	keymap[SCANCODE_I] = SDLK_i;
-	keymap[SCANCODE_O] = SDLK_o;
-	keymap[SCANCODE_P] = SDLK_p;
-	keymap[SCANCODE_BRACKET_LEFT] = SDLK_LEFTBRACKET;
-	keymap[SCANCODE_BRACKET_RIGHT] = SDLK_RIGHTBRACKET;
-	keymap[SCANCODE_ENTER] = SDLK_RETURN;
-	keymap[SCANCODE_LEFTCONTROL] = SDLK_LCTRL;
-	keymap[SCANCODE_A] = SDLK_a;
-	keymap[SCANCODE_S] = SDLK_s;
-	keymap[SCANCODE_D] = SDLK_d;
-	keymap[SCANCODE_F] = SDLK_f;
-	keymap[SCANCODE_G] = SDLK_g;
-	keymap[SCANCODE_H] = SDLK_h;
-	keymap[SCANCODE_J] = SDLK_j;
-	keymap[SCANCODE_K] = SDLK_k;
-	keymap[SCANCODE_L] = SDLK_l;
-	keymap[SCANCODE_SEMICOLON] = SDLK_SEMICOLON;
-	keymap[SCANCODE_APOSTROPHE] = SDLK_QUOTE;
-	keymap[SCANCODE_GRAVE] = SDLK_BACKQUOTE;
-	keymap[SCANCODE_LEFTSHIFT] = SDLK_LSHIFT;
-	keymap[SCANCODE_BACKSLASH] = SDLK_BACKSLASH;
-	keymap[SCANCODE_Z] = SDLK_z;
-	keymap[SCANCODE_X] = SDLK_x;
-	keymap[SCANCODE_C] = SDLK_c;
-	keymap[SCANCODE_V] = SDLK_v;
-	keymap[SCANCODE_B] = SDLK_b;
-	keymap[SCANCODE_N] = SDLK_n;
-	keymap[SCANCODE_M] = SDLK_m;
-	keymap[SCANCODE_COMMA] = SDLK_COMMA;
-	keymap[SCANCODE_PERIOD] = SDLK_PERIOD;
-	keymap[SCANCODE_SLASH] = SDLK_SLASH;
-	keymap[SCANCODE_RIGHTSHIFT] = SDLK_RSHIFT;
-	keymap[SCANCODE_KEYPADMULTIPLY] = SDLK_KP_MULTIPLY;
-	keymap[SCANCODE_LEFTALT] = SDLK_LALT;
-	keymap[SCANCODE_SPACE] = SDLK_SPACE;
-	keymap[SCANCODE_CAPSLOCK] = SDLK_CAPSLOCK;
-	keymap[SCANCODE_F1] = SDLK_F1;
-	keymap[SCANCODE_F2] = SDLK_F2;
-	keymap[SCANCODE_F3] = SDLK_F3;
-	keymap[SCANCODE_F4] = SDLK_F4;
-	keymap[SCANCODE_F5] = SDLK_F5;
-	keymap[SCANCODE_F6] = SDLK_F6;
-	keymap[SCANCODE_F7] = SDLK_F7;
-	keymap[SCANCODE_F8] = SDLK_F8;
-	keymap[SCANCODE_F9] = SDLK_F9;
-	keymap[SCANCODE_F10] = SDLK_F10;
-	keymap[SCANCODE_NUMLOCK] = SDLK_NUMLOCK;
-	keymap[SCANCODE_SCROLLLOCK] = SDLK_SCROLLOCK;
-	keymap[SCANCODE_KEYPAD7] = SDLK_KP7;
-	keymap[SCANCODE_CURSORUPLEFT] = SDLK_KP7;
-	keymap[SCANCODE_KEYPAD8] = SDLK_KP8;
-	keymap[SCANCODE_CURSORUP] = SDLK_KP8;
-	keymap[SCANCODE_KEYPAD9] = SDLK_KP9;
-	keymap[SCANCODE_CURSORUPRIGHT] = SDLK_KP9;
-	keymap[SCANCODE_KEYPADMINUS] = SDLK_KP_MINUS;
-	keymap[SCANCODE_KEYPAD4] = SDLK_KP4;
-	keymap[SCANCODE_CURSORLEFT] = SDLK_KP4;
-	keymap[SCANCODE_KEYPAD5] = SDLK_KP5;
-	keymap[SCANCODE_KEYPAD6] = SDLK_KP6;
-	keymap[SCANCODE_CURSORRIGHT] = SDLK_KP6;
-	keymap[SCANCODE_KEYPADPLUS] = SDLK_KP_PLUS;
-	keymap[SCANCODE_KEYPAD1] = SDLK_KP1;
-	keymap[SCANCODE_CURSORDOWNLEFT] = SDLK_KP1;
-	keymap[SCANCODE_KEYPAD2] = SDLK_KP2;
-	keymap[SCANCODE_CURSORDOWN] = SDLK_KP2;
-	keymap[SCANCODE_KEYPAD3] = SDLK_KP3;
-	keymap[SCANCODE_CURSORDOWNRIGHT] = SDLK_KP3;
-	keymap[SCANCODE_KEYPAD0] = SDLK_KP0;
-	keymap[SCANCODE_KEYPADPERIOD] = SDLK_KP_PERIOD;
-	keymap[SCANCODE_LESS] = SDLK_LESS;
-	keymap[SCANCODE_F11] = SDLK_F11;
-	keymap[SCANCODE_F12] = SDLK_F12;
-	keymap[SCANCODE_KEYPADENTER] = SDLK_KP_ENTER;
-	keymap[SCANCODE_RIGHTCONTROL] = SDLK_RCTRL;
-	keymap[SCANCODE_CONTROL] = SDLK_RCTRL;
-	keymap[SCANCODE_KEYPADDIVIDE] = SDLK_KP_DIVIDE;
-	keymap[SCANCODE_PRINTSCREEN] = SDLK_PRINT;
-	keymap[SCANCODE_RIGHTALT] = SDLK_RALT;
-	keymap[SCANCODE_BREAK] = SDLK_BREAK;
-	keymap[SCANCODE_BREAK_ALTERNATIVE] = SDLK_UNKNOWN;
-	keymap[SCANCODE_HOME] = SDLK_HOME;
-	keymap[SCANCODE_CURSORBLOCKUP] = SDLK_UP;
-	keymap[SCANCODE_PAGEUP] = SDLK_PAGEUP;
-	keymap[SCANCODE_CURSORBLOCKLEFT] = SDLK_LEFT;
-	keymap[SCANCODE_CURSORBLOCKRIGHT] = SDLK_RIGHT;
-	keymap[SCANCODE_END] = SDLK_END;
-	keymap[SCANCODE_CURSORBLOCKDOWN] = SDLK_DOWN;
-	keymap[SCANCODE_PAGEDOWN] = SDLK_PAGEDOWN;
-	keymap[SCANCODE_INSERT] = SDLK_INSERT;
-	keymap[SCANCODE_REMOVE] = SDLK_DELETE;
-	keymap[119] = SDLK_PAUSE;
-	keymap[SCANCODE_RIGHTWIN] = SDLK_RSUPER;
-	keymap[SCANCODE_LEFTWIN] = SDLK_LSUPER;
-	keymap[127] = SDLK_MENU;
+	/* First get the ascii keys and others not well handled */
+	for (i=0; i<SDL_TABLESIZE(keymap); ++i) {
+	  switch(i) {
+	  /* These aren't handled by the x86 kernel keymapping (?) */
+	  case SCANCODE_PRINTSCREEN:
+	    keymap[i] = SDLK_PRINT;
+	    break;
+	  case SCANCODE_BREAK:
+	    keymap[i] = SDLK_BREAK;
+	    break;
+	  case SCANCODE_BREAK_ALTERNATIVE:
+	    keymap[i] = SDLK_PAUSE;
+	    break;
+	  case SCANCODE_LEFTSHIFT:
+	    keymap[i] = SDLK_LSHIFT;
+	    break;
+	  case SCANCODE_RIGHTSHIFT:
+	    keymap[i] = SDLK_RSHIFT;
+	    break;
+	  case SCANCODE_LEFTCONTROL:
+	    keymap[i] = SDLK_LCTRL;
+	    break;
+	  case SCANCODE_RIGHTCONTROL:
+	    keymap[i] = SDLK_RCTRL;
+	    break;
+	  case SCANCODE_RIGHTWIN:
+	    keymap[i] = SDLK_RSUPER;
+	    break;
+	  case SCANCODE_LEFTWIN:
+	    keymap[i] = SDLK_LSUPER;
+	    break;
+	  case 127:
+	    keymap[i] = SDLK_MENU;
+	    break;
+	  /* this should take care of all standard ascii keys */
+	  default:
+	    keymap[i] = KVAL(vga_keymap[0][i]);
+	    break;
+          }
+	}
+	for (i=0; i<SDL_TABLESIZE(keymap); ++i) {
+	  switch(keymap_temp[i]) {
+	    case K_F1:  keymap[i] = SDLK_F1;  break;
+	    case K_F2:  keymap[i] = SDLK_F2;  break;
+	    case K_F3:  keymap[i] = SDLK_F3;  break;
+	    case K_F4:  keymap[i] = SDLK_F4;  break;
+	    case K_F5:  keymap[i] = SDLK_F5;  break;
+	    case K_F6:  keymap[i] = SDLK_F6;  break;
+	    case K_F7:  keymap[i] = SDLK_F7;  break;
+	    case K_F8:  keymap[i] = SDLK_F8;  break;
+	    case K_F9:  keymap[i] = SDLK_F9;  break;
+	    case K_F10: keymap[i] = SDLK_F10; break;
+	    case K_F11: keymap[i] = SDLK_F11; break;
+	    case K_F12: keymap[i] = SDLK_F12; break;
+
+	    case K_DOWN:  keymap[i] = SDLK_DOWN;  break;
+	    case K_LEFT:  keymap[i] = SDLK_LEFT;  break;
+	    case K_RIGHT: keymap[i] = SDLK_RIGHT; break;
+	    case K_UP:    keymap[i] = SDLK_UP;    break;
+
+	    case K_P0:     keymap[i] = SDLK_KP0; break;
+	    case K_P1:     keymap[i] = SDLK_KP1; break;
+	    case K_P2:     keymap[i] = SDLK_KP2; break;
+	    case K_P3:     keymap[i] = SDLK_KP3; break;
+	    case K_P4:     keymap[i] = SDLK_KP4; break;
+	    case K_P5:     keymap[i] = SDLK_KP5; break;
+	    case K_P6:     keymap[i] = SDLK_KP6; break;
+	    case K_P7:     keymap[i] = SDLK_KP7; break;
+	    case K_P8:     keymap[i] = SDLK_KP8; break;
+	    case K_P9:     keymap[i] = SDLK_KP9; break;
+	    case K_PPLUS:  keymap[i] = SDLK_KP_PLUS; break;
+	    case K_PMINUS: keymap[i] = SDLK_KP_MINUS; break;
+	    case K_PSTAR:  keymap[i] = SDLK_KP_MULTIPLY; break;
+	    case K_PSLASH: keymap[i] = SDLK_KP_DIVIDE; break;
+	    case K_PENTER: keymap[i] = SDLK_KP_ENTER; break;
+	    case K_PDOT:   keymap[i] = SDLK_KP_PERIOD; break;
+
+	    case K_SHIFT:  if ( keymap[i] != SDLK_RSHIFT )
+	                     keymap[i] = SDLK_LSHIFT;
+	                   break;
+	    case K_SHIFTL: keymap[i] = SDLK_LSHIFT; break;
+	    case K_SHIFTR: keymap[i] = SDLK_RSHIFT; break;
+	    case K_CTRL:  if ( keymap[i] != SDLK_RCTRL )
+	                     keymap[i] = SDLK_LCTRL;
+	                   break;
+	    case K_CTRLL:  keymap[i] = SDLK_LCTRL;  break;
+	    case K_CTRLR:  keymap[i] = SDLK_RCTRL;  break;
+	    case K_ALT:    keymap[i] = SDLK_LALT;   break;
+	    case K_ALTGR:  keymap[i] = SDLK_RALT;   break;
+
+	    case K_INSERT: keymap[i] = SDLK_INSERT;   break;
+	    case K_REMOVE: keymap[i] = SDLK_DELETE;   break;
+	    case K_PGUP:   keymap[i] = SDLK_PAGEUP;   break;
+	    case K_PGDN:   keymap[i] = SDLK_PAGEDOWN; break;
+	    case K_FIND:   keymap[i] = SDLK_HOME;     break;
+	    case K_SELECT: keymap[i] = SDLK_END;      break;
+
+	    case K_NUM:  keymap[i] = SDLK_NUMLOCK;   break;
+	    case K_CAPS: keymap[i] = SDLK_CAPSLOCK;  break;
+
+	    case K_F13:   keymap[i] = SDLK_PRINT;     break;
+	    case K_HOLD:  keymap[i] = SDLK_SCROLLOCK; break;
+	    case K_PAUSE: keymap[i] = SDLK_PAUSE;     break;
+
+	    case 127: keymap[i] = SDLK_BACKSPACE; break;
+	     
+	    default: break;
+	  }
+	}
 }
 
 static SDL_keysym *TranslateKey(int scancode, SDL_keysym *keysym)
