@@ -1,6 +1,6 @@
 /*
 	SDL - Simple DirectMedia Layer
-	Copyright (C) 1997, 1998, 1999  Sam Lantinga
+	Copyright (C) 1997, 1998, 1999, 2000  Sam Lantinga
 
 	This library is free software; you can redistribute it and/or
 	modify it under the terms of the GNU Library General Public
@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_fbvideo.c,v 1.16 2000/01/25 00:14:41 hercules Exp $";
+ "@(#) $Id: SDL_fbvideo.c,v 1.13.2.12 2000/10/22 20:38:39 hercules Exp $";
 #endif
 
 /* Framebuffer console based SDL video driver implementation.
@@ -32,6 +32,7 @@ static char rcsid =
 #include <stdio.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <sys/ioctl.h>
 #include <sys/mman.h>
 #include <asm/page.h>		/* For definition of PAGE_SIZE */
 
@@ -45,8 +46,14 @@ static char rcsid =
 #include "SDL_fbvideo.h"
 #include "SDL_fbmouse_c.h"
 #include "SDL_fbevents_c.h"
+#include "SDL_fb3dfx.h"
 #include "SDL_fbmatrox.h"
 
+/* Fix for broken 2.0.4-test1-ruby fb.h */
+typedef struct fb_var_screeninfo fb_var_screeninfo;
+typedef struct fb_fix_screeninfo fb_fix_screeninfo;
+typedef struct fb_cmap fb_cmap;
+
 /* A list of video resolutions that we query for (sorted largest to smallest) */
 static SDL_Rect checkres[] = {
 	{  0, 0, 1600, 1200 },		/* 16 bpp: 0x11E, or 286 */
@@ -112,7 +119,7 @@ static struct {
 static int FB_VideoInit(_THIS, SDL_PixelFormat *vformat);
 static SDL_Rect **FB_ListModes(_THIS, SDL_PixelFormat *format, Uint32 flags);
 static SDL_Surface *FB_SetVideoMode(_THIS, SDL_Surface *current, int width, int height, int bpp, Uint32 flags);
-static int FB_SetColors(_THIS, int firstcolor, int ncolors);
+static int FB_SetColors(_THIS, int firstcolor, int ncolors, SDL_Color *colors);
 static void FB_VideoQuit(_THIS);
 
 /* Hardware surface functions */
@@ -302,7 +309,6 @@ static int FB_VideoInit(_THIS, SDL_PixelFormat *vformat)
 	int current_index;
 	unsigned int current_w;
 	unsigned int current_h;
-	int keyboard;
 	const char *SDL_fbdev;
 
 	/* Initialize the library */
@@ -438,7 +444,7 @@ static int FB_VideoInit(_THIS, SDL_PixelFormat *vformat)
 		    case FB_ACCEL_MATROX_MGA2164W:
 		    case FB_ACCEL_MATROX_MGA2164W_AGP:
 		    case FB_ACCEL_MATROX_MGAG100:
-		    case FB_ACCEL_MATROX_MGAG200:
+		    /*case FB_ACCEL_MATROX_MGAG200: G200 acceleration broken! */
 		    case FB_ACCEL_MATROX_MGAG400:
 #ifdef FBACCEL_DEBUG
 			printf("Matrox hardware accelerator!\n");
@@ -600,7 +606,6 @@ SDL_Surface *FB_SetVideoMode(_THIS, SDL_Surface *current,
 	fprintf(stderr, "Printing actual vinfo:\n");
 	print_vinfo(&vinfo);
 #endif
-	free(current->format);
 	Rmask = 0;
 	for ( i=0; i<vinfo.red.length; ++i ) {
 		Rmask <<= 1;
@@ -616,9 +621,8 @@ SDL_Surface *FB_SetVideoMode(_THIS, SDL_Surface *current,
 		Bmask <<= 1;
 		Bmask |= (0x00000001<<vinfo.blue.offset);
 	}
-	current->format = SDL_AllocFormat(vinfo.bits_per_pixel,
-	                                  Rmask, Gmask, Bmask, 0);
-	if ( current->format == NULL ) {
+	if ( ! SDL_ReallocFormat(current, vinfo.bits_per_pixel,
+	                                  Rmask, Gmask, Bmask, 0) ) {
 		return(NULL);
 	}
 
@@ -640,6 +644,15 @@ SDL_Surface *FB_SetVideoMode(_THIS, SDL_Surface *current,
 	current->pitch = finfo.line_length;
 	current->pixels = mapped_mem+mapped_offset;
 
+	/* Let the application know we have a hardware palette */
+	switch (finfo.visual) {
+	    case FB_VISUAL_PSEUDOCOLOR:
+		current->flags |= SDL_HWPALETTE;
+		break;
+	    default:
+		break;
+	}
+
 	/* Update for double-buffering, if we can */
 	if ( flags & SDL_DOUBLEBUF ) {
 		if ( vinfo.yres_virtual == (height*2) ) {
@@ -704,6 +717,7 @@ static int FB_InitHWSurfaces(_THIS, caddr_t base, int size)
 	surfaces.next = NULL;
 	surfaces_memtotal = size;
 	surfaces_memleft = size;
+	return(0);
 }
 static void FB_FreeHWSurfaces(_THIS)
 {
@@ -715,6 +729,7 @@ static void FB_FreeHWSurfaces(_THIS)
 		bucket = bucket->next;
 		free(freeable);
 	}
+	surfaces.next = NULL;
 }
 
 static int FB_AllocHWSurface(_THIS, SDL_Surface *surface)
@@ -960,21 +975,19 @@ static void FB_RestorePalette(_THIS)
 	}
 }
 
-static int FB_SetColors(_THIS, int firstcolor, int ncolors)
+static int FB_SetColors(_THIS, int firstcolor, int ncolors, SDL_Color *colors)
 {
-	SDL_Color *colors;
-	int i, which;
+	int i;
 	__u16 r[256];
 	__u16 g[256];
 	__u16 b[256];
 	struct fb_cmap cmap;
 
 	/* Set up the colormap */
-	colors = this->screen->format->palette->colors;
-	for ( i=ncolors-1, which=firstcolor+ncolors-1; i>=0; --i, --which ) {
-		r[i] = ((__u16)colors[which].r)<<8;
-		g[i] = ((__u16)colors[which].g)<<8;
-		b[i] = ((__u16)colors[which].b)<<8;
+	for (i = 0; i < ncolors; i++) {
+		r[i] = colors[i].r << 8;
+		g[i] = colors[i].g << 8;
+		b[i] = colors[i].b << 8;
 	}
 	cmap.start = firstcolor;
 	cmap.len = ncolors;
@@ -983,7 +996,9 @@ static int FB_SetColors(_THIS, int firstcolor, int ncolors)
 	cmap.blue = b;
 	cmap.transp = NULL;
 
-	if ( ioctl(console_fd, FBIOPUTCMAP, &cmap) < 0 ) {
+	if( (ioctl(console_fd, FBIOPUTCMAP, &cmap) < 0) &&
+	    !(this->screen->flags & SDL_HWPALETTE) ) {
+	        colors = this->screen->format->palette->colors;
 		ncolors = this->screen->format->palette->ncolors;
 		cmap.start = 0;
 		cmap.len = ncolors;
