@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_svgaevents_c.h,v 1.2 1999/12/17 19:10:43 hercules Exp $";
+ "@(#) $Id: SDL_svgaevents_c.h,v 1.2.2.2 2000/10/03 16:00:58 hercules Exp $";
 #endif
 
 #include "SDL_svgavideo.h"
@@ -30,7 +30,7 @@ static char rcsid =
 /* Variables and functions exported by SDL_sysevents.c to other parts 
    of the native video subsystem (SDL_sysvideo.c)
 */
-extern void SVGA_initkeymaps(int fd);
+extern int SVGA_initkeymaps(int fd);
 extern void SVGA_mousecallback(int button, int dx, int dy,
                                  int u1,int u2,int u3, int u4);
 extern void SVGA_keyboardcallback(int scancode, int pressed);
