@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_x11image.c,v 1.2 2000/01/16 14:34:22 hercules Exp $";
+ "@(#) $Id: SDL_x11image.c,v 1.1.2.11 2000/10/22 00:31:16 hercules Exp $";
 #endif
 
 #include <stdlib.h>
@@ -31,6 +31,10 @@ static char rcsid =
 #include "SDL_endian.h"
 #include "SDL_x11image_c.h"
 
+#ifdef HAVE_KSTAT
+#include <kstat.h>
+#endif
+
 /* Shared memory information */
 extern int XShmQueryExtension(Display *dpy);	/* Not in X11 headers */
 
@@ -89,16 +93,17 @@ int X11_SetupImage(_THIS, SDL_Surface *screen)
 
 	if ( use_mitshm ) {
 		SDL_Ximage = XShmCreateImage(SDL_Display, SDL_Visual,
-			DefaultDepth(SDL_Display, SDL_Screen),
-			ZPixmap, shminfo.shmaddr, &shminfo, 
-						screen->w, screen->h);
+					     this->hidden->depth, ZPixmap,
+					     shminfo.shmaddr, &shminfo, 
+					     screen->w, screen->h);
 	} else {
+ 	        int bpp = screen->format->BytesPerPixel;
 		SDL_Ximage = XCreateImage(SDL_Display, SDL_Visual,
-			DefaultDepth(SDL_Display, SDL_Screen),
-			ZPixmap, 0, (char *)screen->pixels, 
-			screen->w, screen->h,
-			((screen->format)->BytesPerPixel == 3) ? 32 :
-				(screen->format)->BytesPerPixel*8, 0);
+					  this->hidden->depth, ZPixmap, 0,
+					  (char *)screen->pixels, 
+					  screen->w, screen->h,
+					  (bpp == 3) ? 32 : bpp * 8,
+					  0);
 	}
 	if ( SDL_Ximage == NULL ) {
 		SDL_SetError("Couldn't create XImage");
@@ -136,12 +141,63 @@ void X11_DestroyImage(_THIS, SDL_Surface *screen)
 	}
 }
 
+/* This is a hack to see whether this system has more than 1 CPU */
+static int num_CPU(void)
+{
+       static int num_cpus = 0;
+
+       if(!num_cpus) {
+#ifdef linux
+           char line[BUFSIZ];
+           FILE *cpuinfo = fopen("/proc/cpuinfo", "r");
+           if ( cpuinfo ) {
+               while ( fgets(line, sizeof(line)-1, cpuinfo) ) {
+                   if (!strncmp(line, "processor", strlen("processor"))) {
+                       ++num_cpus;
+                   }
+               }
+               fclose(cpuinfo);
+           }
+#elif defined(HAVE_KSTAT)
+           kstat_ctl_t *kc = kstat_open();
+           kstat_t *ks;
+           kstat_named_t *kn;
+           if(kc) {
+               if((ks = kstat_lookup(kc, "unix", -1, "system_misc"))
+                  && kstat_read(kc, ks, NULL) != -1
+                  && (kn = kstat_data_lookup(ks, "ncpus")))
+                   num_cpus = kn->value.i32;
+               kstat_close(kc);
+           }
+#endif
+           if ( num_cpus == 0 ) {
+               num_cpus = 1;
+           }
+       }
+       return num_cpus;
+}
+
 int X11_ResizeImage(_THIS, SDL_Surface *screen, Uint32 flags)
 {
 	int retval;
 
 	X11_DestroyImage(this, screen);
-	retval = X11_SetupImage(this, screen);
+        if ( flags & SDL_OPENGL ) {  /* No image when using GL */
+        	retval = 0;
+        } else {
+		retval = X11_SetupImage(this, screen);
+		/* We support asynchronous blitting on the display */
+		if ( flags & SDL_ASYNCBLIT ) {
+			/* This is actually slower on single-CPU systems,
+			   probably because of CPU contention between the
+			   X server and the application.
+			   Note: Is this still true with XFree86 4.0?
+			*/
+			if ( num_CPU() > 1 ) {
+				screen->flags |= SDL_ASYNCBLIT;
+			}
+		}
+	}
 	return(retval);
 }
 
@@ -157,6 +213,10 @@ void X11_FreeHWSurface(_THIS, SDL_Surface *surface)
 
 int X11_LockHWSurface(_THIS, SDL_Surface *surface)
 {
+	if ( (surface == SDL_VideoSurface) && blit_queued ) {
+		XSync(GFX_Display, False);
+		blit_queued = 0;
+	}
 	return(0);
 }
 void X11_UnlockHWSurface(_THIS, SDL_Surface *surface)
@@ -170,26 +230,18 @@ int X11_FlipHWSurface(_THIS, SDL_Surface *surface)
 }
 
 /* Byte-swap the pixels in the display image */
-static void X11_SwapPixels(SDL_Surface *screen, int numrects, SDL_Rect *rects)
+static void X11_SwapAllPixels(SDL_Surface *screen)
 {
-	int i;
-	int x, minx, maxx;
-	int y, miny, maxy;
+	int x, y;
 
 	switch (screen->format->BytesPerPixel) {
 	    case 2: {
 		Uint16 *spot;
-		for ( i=0; i<numrects; ++i ) {
-			minx = rects[i].x;
-			maxx = rects[i].x+rects[i].w;
-			miny = rects[i].y;
-			maxy = rects[i].y+rects[i].h;
-			for ( y=miny; y<maxy; ++y ) {
-				spot = (Uint16 *) ((Uint8 *)screen->pixels +
-						y * screen->pitch + minx * 2);
-				for ( x=minx; x<maxx; ++x, ++spot ) {
-					*spot = SDL_Swap16(*spot);
-				}
+		for ( y=0; y<screen->h; ++y ) {
+			spot = (Uint16 *) ((Uint8 *)screen->pixels +
+						y * screen->pitch);
+			for ( x=0; x<screen->w; ++x, ++spot ) {
+				*spot = SDL_Swap16(*spot);
 			}
 		}
 	    }
@@ -197,17 +249,11 @@ static void X11_SwapPixels(SDL_Surface *screen, int numrects, SDL_Rect *rects)
 
 	    case 4: {
 		Uint32 *spot;
-		for ( i=0; i<numrects; ++i ) {
-			minx = rects[i].x;
-			maxx = rects[i].x+rects[i].w;
-			miny = rects[i].y;
-			maxy = rects[i].y+rects[i].h;
-			for ( y=miny; y<maxy; ++y ) {
-				spot = (Uint32 *) ((Uint8 *)screen->pixels +
-						y * screen->pitch + minx * 4);
-				for ( x=minx; x<maxx; ++x, ++spot ) {
-					*spot = SDL_Swap32(*spot);
-				}
+		for ( y=0; y<screen->h; ++y ) {
+			spot = (Uint32 *) ((Uint8 *)screen->pixels +
+						y * screen->pitch);
+			for ( x=0; x<screen->w; ++x, ++spot ) {
+				*spot = SDL_Swap32(*spot);
 			}
 		}
 	    }
@@ -218,6 +264,49 @@ static void X11_SwapPixels(SDL_Surface *screen, int numrects, SDL_Rect *rects)
 		break;
 	}
 }
+static void X11_SwapPixels(SDL_Surface *screen, SDL_Rect *rect)
+{
+	int x, minx, maxx;
+	int y, miny, maxy;
+
+	switch (screen->format->BytesPerPixel) {
+	    case 2: {
+		Uint16 *spot;
+		minx = rect->x;
+		maxx = rect->x + rect->w;
+		miny = rect->y;
+		maxy = rect->y + rect->h;
+		for ( y=miny; y<maxy; ++y ) {
+		    spot = (Uint16 *) ((Uint8 *)screen->pixels +
+				       y * screen->pitch + minx * 2);
+		    for ( x=minx; x<maxx; ++x, ++spot ) {
+			*spot = SDL_Swap16(*spot);
+		    }
+		}
+	    }
+	    break;
+
+	    case 4: {
+		Uint32 *spot;
+		minx = rect->x;
+		maxx = rect->x + rect->w;
+		miny = rect->y;
+		maxy = rect->y + rect->h;
+		for ( y=miny; y<maxy; ++y ) {
+		    spot = (Uint32 *) ((Uint8 *)screen->pixels +
+				       y * screen->pitch + minx * 4);
+		    for ( x=minx; x<maxx; ++x, ++spot ) {
+			*spot = SDL_Swap32(*spot);
+		    }
+		}
+	    }
+	    break;
+
+	    default:
+		/* should never get here */
+		break;
+	}
+}
 
 static void X11_NormalUpdate(_THIS, int numrects, SDL_Rect *rects)
 {
@@ -226,21 +315,32 @@ static void X11_NormalUpdate(_THIS, int numrects, SDL_Rect *rects)
 	/* Check for endian-swapped X server, swap if necessary (VERY slow!) */
 	if ( swap_pixels &&
 	     ((this->screen->format->BytesPerPixel%2) == 0) ) {
-		X11_SwapPixels(this->screen, numrects, rects);
 		for ( i=0; i<numrects; ++i ) {
+			if ( ! rects[i].w ) { /* Clipped? */
+				continue;
+			}
+		        X11_SwapPixels(this->screen, rects + i);
 			XPutImage(GFX_Display, SDL_Window, SDL_GC, SDL_Ximage,
 				rects[i].x, rects[i].y,
 				rects[i].x, rects[i].y, rects[i].w, rects[i].h);
+			X11_SwapPixels(this->screen, rects + i);
 		}
-		X11_SwapPixels(this->screen, numrects, rects);
 	} else {
 		for ( i=0; i<numrects; ++i ) {
+			if ( ! rects[i].w ) { /* Clipped? */
+				continue;
+			}
 			XPutImage(GFX_Display, SDL_Window, SDL_GC, SDL_Ximage,
 				rects[i].x, rects[i].y,
 				rects[i].x, rects[i].y, rects[i].w, rects[i].h);
 		}
 	}
-	XSync(GFX_Display, False);
+	if ( SDL_VideoSurface->flags & SDL_ASYNCBLIT ) {
+		XFlush(GFX_Display);
+		++blit_queued;
+	} else {
+		XSync(GFX_Display, False);
+	}
 }
 
 static void X11_MITSHMUpdate(_THIS, int numrects, SDL_Rect *rects)
@@ -248,11 +348,44 @@ static void X11_MITSHMUpdate(_THIS, int numrects, SDL_Rect *rects)
 	int i;
 
 	for ( i=0; i<numrects; ++i ) {
+		if ( ! rects[i].w ) { /* Clipped? */
+			continue;
+		}
 		XShmPutImage(GFX_Display, SDL_Window, SDL_GC, SDL_Ximage,
 				rects[i].x, rects[i].y,
 				rects[i].x, rects[i].y, rects[i].w, rects[i].h,
 									False);
 	}
-	XSync(GFX_Display, False);
+	if ( SDL_VideoSurface->flags & SDL_ASYNCBLIT ) {
+		XFlush(GFX_Display);
+		++blit_queued;
+	} else {
+		XSync(GFX_Display, False);
+	}
 }
 
+void X11_RefreshDisplay(_THIS)
+{
+	/* Don't refresh a display that doesn't have an image (like GL) */
+	if ( ! SDL_Ximage ) {
+		return;
+	}
+	if ( this->UpdateRects == X11_MITSHMUpdate ) {
+		XShmPutImage(SDL_Display, SDL_Window, SDL_GC, SDL_Ximage,
+				0, 0, 0, 0, this->screen->w, this->screen->h,
+				False);
+	} else {
+		/* Check for endian-swapped X server, swap if necessary */
+		if ( swap_pixels &&
+		     ((this->screen->format->BytesPerPixel%2) == 0) ) {
+			X11_SwapAllPixels(this->screen);
+			XPutImage(SDL_Display, SDL_Window, SDL_GC, SDL_Ximage,
+				0, 0, 0, 0, this->screen->w, this->screen->h);
+			X11_SwapAllPixels(this->screen);
+		} else {
+			XPutImage(SDL_Display, SDL_Window, SDL_GC, SDL_Ximage,
+				0, 0, 0, 0, this->screen->w, this->screen->h);
+		}
+	}
+	XSync(SDL_Display, False);
+}
